package influence;

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class InfluenceMap extends java.awt.Panel implements Serializable{
    private int w = 30;
    private int h = 30;
    transient Image buffer;
    
    Cell cells[][] = new Cell[w][h];
    
    Unit next = new Unit(new Point(0,0),5,5,true);
    
    public InfluenceMap()
	{
		super();
		//{{INIT_CONTROLS
		setLayout(new GridLayout(w,h));
		setSize(insets().left + insets().right + 430,insets().top + insets().bottom + 270);
		//}}
        for(int i=0;i<w;i++){
            for(int j=0;j<h;j++){
               cells[j][i] = new Cell();
               cells[j][i].setPosition(new Point(j,i));
               add(cells[j][i]);
               cells[j][i].addMouseListener(new MouseAdapter(){public void mouseClicked(MouseEvent e){addUnitAt(((Cell)e.getSource()).getPosition());}});
            }
        }
    }
    
    public void update(Graphics g){
        paint(g);
    }
    
    public void paint(Graphics g){
    if(buffer == null){buffer = createImage(getBounds().width,getBounds().height);}
        Graphics bg = buffer.getGraphics();
        super.paint(bg);
        bg.setColor(Color.black);
        g.drawImage(buffer,0,0,null);
    }
   
    public void clearMap(){
        for(int i=0;i<w;i++){
            for(int j=0;j<h;j++){
               cells[j][i].setLevel(0);
             
            }
        }
        repaint();
    }
        
   
    public void setUnitType(Unit unit){
        next = unit;
    }
    
    public void addUnitAt(Point p){
        next.setPosition(p);
        addUnit(next);
    }
    
    public void addUnit(Unit unit){
        double delta;
        double dist;
        int dev = unit.getDeviation();
        int str = unit.getInfluence();
        Point p = unit.getPosition();
        
        //bounds check, add OffMapException later
        int min_x,min_y,max_x,max_y;
        
        min_x = p.x -3 * dev;
        max_x = p.x +3 * dev;
        min_y = p.y -3 * dev;
        max_y = p.y +3 * dev;
        
        min_x = (min_x>0)?min_x:0;
        max_x  =(max_x<w)?max_x:w-1;
        min_y = (min_y>0)?min_y:0;
        max_y  =(max_y<h)?max_y:h-1;
        
        for(int i=min_x;i<=max_x;i++){
            for(int j=min_y;j<=max_y;j++){
                dist = Math.sqrt((Math.pow(p.x-i,2)+Math.pow(p.y-j,2)));
                if(dist<dev){
                   delta = 1.0 * str - ((0.1 * str * dist) / dev);
                }
                else if (dist <= 2* dev){
                    delta = 0.9 * str - ((0.8 * str * (dist - dev)));
                }
                else if (dist <=3*dev) {
                    delta = 0.1 * str - ((0.1 * str *(dist - 2*dev)));
                }
                else{
                    continue;
                }
                if(unit.isSideA()){
                    delta = Math.max(delta,0);
                  
                }
                else{
                    delta = Math.min(delta,0);
                   
                }
                cells[i][j].changeLevel((int)delta);
            }
        }
        repaint();            
                
                  
        
    }
    
}